#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (C) 2008 Sergey Poznyakoff
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from wikimarkup import *
from types import TupleType
import urllib

class HtmlWikiMarkup (WikiMarkup):
    """
    A (hopefully) general-purpose Wiki->HTML translator class.
    FIXME: 1. See WikiMarkup for a list
           2. [[official position]]s : final 's' gets after closing </a> tag.
           Should be before.
    """

    # FIXME: Awful kludge
    image_kw = [ 'Image',
                 'Grafika',
                 'Bild',
                 'Εικόνα',
                 'Dosiero',
                 'Slika',
                 'Resim'
                 ]
    
    ST_INIT = 0
    ST_PARA = 1
    ST_OPEN = 2
    
    state = []

    def opara(self):
        if self.state[-1] == self.ST_PARA:
            self.state[-1] = self.ST_OPEN
            return "<p>"
        else:
            return ""
        
    def cpara(self):
        state = self.state.pop();
        self.state.append(self.ST_INIT)
        if state == self.ST_OPEN:
            return "</p>"
        else:
            return ""
    
            
    def target(self, t):
        (qual,sep,tgt) = t.partition(':')
        r = None
        if tgt != '':
            if qual in self.image_kw:
                t = self.image_base + urllib.quote(tgt) + '/250px-' + urllib.quote(tgt)
            elif qual == "Media":
                t = self.media_base + '/' + tgt
            elif qual in self.langtab:
                t = self.html_base % { 'lang' : qual } + '/' + urllib.quote(tgt)
                r = self.langtab[qual]
            else:
                t = self.html_base % { 'lang' : self.lang } + '/' + urllib.quote(t)
        else:
            t = self.html_base % { 'lang' : self.lang } + '/' + urllib.quote(t)
        return t, r

    envhdr = [ "ul", "ol", "dl" ]    
    envel = [ "li", "li", "dd" ]

    def str_link(self, tok, env):
        arg = self.fmtok(tok[1], env)
        (target, r) = self.target(arg)
        text = self.fmtok(tok[2], env)
        return "<a href=\"%s\">%s</a>" % (target,
                                          text if (text and text != '') \
                                               else \
                                              r if r else arg)

    def str_tmpl(self, tok, env):
        arg = self.fmtok(tok[1], env)
        (target, r) = self.target(arg)
        text = self.fmtok(tok[2], env)
        return "<a href=\"%s\">%s</a>" % (target,
                                          text if (text and text != '') \
                                               else arg)

    def str_ref(self, tok, env):
        target = self.fmtok(tok[1], env)
        text = self.fmtok(tok[2], env)
        return "<a href=\"%s\">%s</a>" % (target,
                                          text if (text and text != '') \
                                                   else target)
    def str_it(self, tok, env):
        return "<i>" + self.fmtok(tok[1], env) + "</i>"
                                          
    def str_bold(self, tok, env):
        return "<b>" + self.fmtok(tok[1], env) + "</b>"
                                              
    def str_hdr(self, tok, env):
        level = tok[1]
        if level > 4:
            level = 4
        return "<h%s>%s</h%s>" % (level, self.fmtok(tok[2], env), level)
    
    def str_bar(self, tok, env):
        return "<hr/>"
    
    def str_env(self, tok, env):
        t = tok[1]
        return "<" + self.envhdr[t] + ">" + \
            self.fmtok(tok[3], tok) + \
            "</" + self.envhdr[t] + ">"
    
    def str_item(self, tok, env):
        return "<%s>%s</%s>" % (self.envel[env[1]],
                        self.fmtok(tok[1], env),
                        self.envel[env[1]])

    def str_para(self, tok, env):
        s = self.cpara()
        self.state.append(self.ST_PARA)
        return s

    def fmtok(self, tok, env):
        if type(tok) != TupleType:
            return ""
        if tok[0] in [ self.ENV, self.HDR ]:
            s = self.cpara()
        elif tok[0] == self.BAR:
            s = self.str_para(tok, env)
        elif tok[0] in [ self.NIL, self.SEQ ]:
            s = ""
        else:
            s = self.opara()
        s1 = WikiMarkup.fmtok(self, tok, env)
        if s1:
            s += s1
        return s
    
    def __str__(self):
        self.state = [ self.ST_PARA ]
        return WikiMarkup.__str__(self) + self.cpara()



class HtmlWiktionaryMarkup (HtmlWikiMarkup):
    """
 A class for translating Wiktionary articles into HTML.
 This version does not do much, except that it tries to correctly
 format templates. But "tries" does not mean "does". The heuristics
 used here is clearly not enough to cope with it.

 1. FIXME:    
 The right solution would be to have a database of templates with their
 semantics and to decide on their rendering depending on that. E.g.
 {{term}} in en.wiktionary means "replace this with the search term".
 This, however, does not work in other wiktionaries. There are
 also more complex templates, e.g.: {{t+|bg|врата|n|p|tr=vrata|sc=Cyrl}}
 I don't know what it means. Couldn't find any documentation either.
 Again, this template does not work in other dictionaries.

 2. Capitulation notice:    
 Given the:
   1. waste amount of wiktionaries available,
   2. abundance of various templates for each wictionary,
   3. apparent lack of documentation thereof,
   4. the lack of standardized language-independent templates,
 I dont see any way to cope with the template-rendering task within a
 reasonable amount of time.
 
 Faeci quod potui, faciant meliora potentes.    
    """
    seq_pos = 0
    
    def str_seq(self, tok, env):
            str = ""
            self.seq_pos=0
            for t in tok[1:]:
                s = self.fmtok(t, env)
                if s:
                    str += s
                    self.seq_pos += 1
            return str
    
    def str_tmpl(self, tok, env):
        arg = self.fmtok(tok[1], env)
        if self.seq_pos > 0:
            return " <b>" + arg + "</b>"
        else:
            return "<br/><b>" + arg + ":</b><br/>"
                
